/***************************************************************
From the book "Motif Programming, The Essentials...and More"
by Marshall Brain.
Published by Digital Press, ISBN 1-55558-089-0.
To order the book, call 1-800-DIGITAL and ask for EY-J816E-DP.

Copyright 1992, by Digital Equipment Corp.

This code demonstrates the use of the Motif clipboard.
***************************************************************/

/* clipboard_test.c */

#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/CutPaste.h>

XtAppContext context;
XmStringCharSet char_set=XmSTRING_DEFAULT_CHARSET;

Widget toplevel;
Widget button;

void show_status(status,s)
    int status;
    char *s;
{
    switch (status)
    {
        case ClipboardSuccess:
            printf("%s successful\n",s);
            break;
        case ClipboardLocked:
            printf("%s found locked clipboard\n",s);
            break;
        case ClipboardFail:
            printf("%s failed\n",s);
            break;
        case ClipboardTruncate:
            printf("%s truncated data\n",s);
            break;
        case ClipboardNoData:
            printf("%s found no data\n",s);
            break;
    }
}

void buttonCB(w,client_data,call_data)
    Widget w;
    XtPointer client_data;
    XmPushButtonCallbackStruct *call_data;
/* called whenever pushbutton is clicked. */
/* gets the contents of the clipboard and dumps it to stdout. */
{
    char format[1000];
    char buffer[10000];
    int status, private_id;
    unsigned long num_bytes;

    /* get the first format of the clipboard. */
    status=XmClipboardInquireFormat(XtDisplay(toplevel), XtWindow(w), 
        1, format, 999, &num_bytes);
    format[num_bytes]='\0';
    show_status(status,"XmClipboardInquireFormat");
    printf("Format=%s\n",format);

    /* start the retrieve. */
    status=XmClipboardStartRetrieve(XtDisplay(toplevel), XtWindow(w), 
        call_data->event->xbutton.time);
    show_status(status,"XmClipboardStartRetrieve");

    /* get the data and dump it. */
    status=XmClipboardRetrieve(XtDisplay(toplevel), XtWindow(w),
        format, buffer, 9999, &num_bytes, &private_id); 
    buffer[num_bytes]='\0';
    show_status(status,"XmClipboardRetrieve");
    printf("format=%s\n",format);
    printf("Private ID=%d\n",private_id);
    printf("num of bytes=%d\n",num_bytes);       
    printf("data=%s\n",buffer);

    /* end the retrieve. */
    status=XmClipboardEndRetrieve(XtDisplay(toplevel), XtWindow(w)); 
    show_status(status,"XmClipboardEndRetrieve");
    printf("----------------------------------------\n");
}

void main(argc,argv)
    int argc;
    char *argv[];
{
    Arg al[10];
    int ac;

    /* create the toplevel shell */
    toplevel = XtAppInitialize(&context,"",NULL,0,&argc,argv,
        NULL,NULL,0);

    /* create a push button */
    ac=0;
    XtSetArg(al[ac],XmNlabelString,XmStringCreate
        ("Push to dump clipboard contents",char_set)); ac++;
    button=XmCreatePushButton(toplevel,"button",al,ac);
    XtManageChild(button);
    XtAddCallback(button,XmNactivateCallback,buttonCB,NULL);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(context);
}
